try:
	from .newcore import JiVi,Thread
except:
	from newcore import JiVi,Thread

J 	 = JiVi
JC   = JiVi.ALL.JiviCore
JAr  = JiVi.ALL.JiviAr
JD 	 = JiVi.ALL.JiviDir
JF   = JiVi.ALL.JiviFile
JIO  = JiVi.ALL.JiviIO
JS   = JiVi.ALL.JiviStr

