SERIALIZE_NAME
BGCOLOR
COLUMN_LABEL
CONTEXT_HELP_ID



FONT

DCOLOR




FGCOLOR




FORMAT




DECIMALS

INITIAL







LABEL




MOUSE_POINTER

PFCOLOR

BEFORE_TABLE


















































