import jivi.util as jvUtil
from jivi.wrapper import jvWrapper,jvWrapperSuper
import jivi.menu as jvMenu
import jivi.fs as jvFs



class jv:
	menu = jvMenu
	wrapper = jvWrapper
	util = jvUtil
	fs = jvFs
	